import os, unittest
from binascii import hexlify
from pure25519.spake2 import start_U, finish_U, start_V, finish_V, U, V

class SPAKE2(unittest.TestCase):
    def assertBytesEqual(self, e1, e2):
        self.assertEqual(hexlify(e1), hexlify(e2))

    def test_success(self):
        pw = b"password"
        sd_U,X = start_U(pw, os.urandom, b"idA", b"idB")
        sd_V,Y = start_V(pw, os.urandom, b"idA", b"idB")
        K1 = finish_U(sd_U,Y)
        K2 = finish_V(sd_V,X)
        self.assertBytesEqual(K1, K2)

    def test_failure(self):
        sd_U,X = start_U(b"password", os.urandom, b"idA", b"idB")
        sd_V,Y = start_V(b"wrong", os.urandom, b"idA", b"idB")
        K1 = finish_U(sd_U,Y)
        K2 = finish_V(sd_V,X)
        self.assertNotEqual(hexlify(K1), hexlify(K2))

    def test_blinding_factors(self):
        self.assertEqual(hexlify(U.to_bytes()).decode("ascii"), expected_U)
        self.assertEqual(hexlify(V.to_bytes()).decode("ascii"), expected_V)

expected_U = "6d7107929f9fb8ddeb0788f4bd6cd0a39b5cdcf71b03c41029aae74eda5f64f3"
expected_V = "48032d6a3406ad4860cca367750cea4f26ea14265ad57ffc6aefe9bf68994054"



import timeit

def do(setup_statements, statement):
    # extracted from timeit.py
    t = timeit.Timer(stmt=statement,
                     setup="\n".join(setup_statements))
    # determine number so that 0.2 <= total time < 2.0
    for i in range(1, 10):
        number = 10**i
        x = t.timeit(number)
        if x >= 1.0:
            break
    return x / number

def abbrev(t):
    if t > 1.0:
        return "%.3fs" % t
    if t > 1e-3:
        return "%.2fms" % (t*1e3)
    if t > 1e-6:
        return "%.2fus" % (t*1e6)
    return "%.2fns" % (t*1e9)

def p(name, setup_statements, statements):
    t = sorted([do(setup_statements, statements) for i in range(3)])
    print("%-32s: %s (%s)" % (name,
                             abbrev(min(t)),
                             " ".join([abbrev(s) for s in t])))

# pure_ed25519.sign() is doing an extra publickey(), doubles the cost

def run():
    S1 = "from pure25519 import basic, slow_basic"
    S2 = "p=slow_basic.scalarmult_affine(basic.B, 16*1000000000)"
    S3 = "P=basic.encodepoint(p)"
    S4 = "basic.decodepoint(P)"
    S5big =   r"i = b'\xf0'+b'\xff'*31"
    S5medium =   r"i = b'\xf0'+b'\x55'*31"
    S5small = r"i = b'\xf0'+b'\x00'*31"
    S6 = "si=basic.bytes_to_scalar(i)"
    S7 = "basic.scalar_to_bytes(si)"
    S8 = "slow_basic.slow_scalarmult_affine(p, si)"
    S9 = "slow_basic.scalarmult_affine(p, si)"
    S10 = "x=basic.xform_affine_to_extended(p)"
    S11 = "basic.xform_extended_to_affine(x)"
    S12 = "p2 = slow_basic.scalarmult_affine(basic.B, 17)"
    S13 = "slow_basic.slow_add_affine(p, p2)"
    S14 = "x2=basic.xform_affine_to_extended(p2)"
    S15 = "basic.add_elements(x, x2)"
    S16 = "basic.xrecover(basic.Bx)"
    S17 = "basic.isoncurve(p)"
    S18 = "(si + si) % basic.Q"
    S19 = "(si * si) % basic.Q"
    S20 = "basic.inv(si)"
    S21 = "basic._add_elements_nonunfied(x, x2)"
    S22 = "e=basic.bytes_to_unknown_group_element(P)"
    S23 = "e=basic.bytes_to_element(P)"
    S24 = "e=basic.arbitrary_element(b'seed')"
    S25 = "e.scalarmult(si)"

    print("speed_basic")
    if 1:
        p("encodepoint", [S1,S2], S3)
        p("decodepoint", [S1,S2,S3], S4)
        p("scalar_to_bytes", [S1,S5big,S6], S7)
        p("bytes_to_scalar", [S1,S5big], S6)
        p("slow_scalarmult_affine (big)", [S1,S2,S5big,S6], S8)
        p("slow_scalarmult_affine (medium)", [S1,S2,S5medium,S6], S8)
        p("slow_scalarmult_affine (small)", [S1,S2,S5small,S6], S8)
        p("scalarmult_affine (big)", [S1,S2,S5big,S6], S9)
        p("scalarmult_affine (medium)", [S1,S2,S5medium,S6], S9)
        p("scalarmult_affine (small)", [S1,S2,S5small,S6], S9)
        p("xform_affine_to_extended", [S1,S2], S10)
        p("xform_extended_to_affine", [S1,S2,S10], S11)
        p("slow_add_affine", [S1,S2,S12], S13)
        p("add (extended)", [S1,S2,S12,S10,S14], S15)
        p("add_nonunified (extended)", [S1,S2,S12,S10,S14], S21)
        p("xrecover", [S1], S16)
        p("isoncurve", [S1,S2], S17)
    if 1:
        p("field_add", [S1,S5medium,S6], S18)
        p("field_mul (big)", [S1,S5big,S6], S19)
        p("field_mul (medium)", [S1,S5medium,S6], S19)
        p("field_mul (small)", [S1,S5small,S6], S19)
        p("field_inv", [S1,S5medium,S6], S20)
    if 1:
        p("bytes_to_unknown_group_element", [S1,S2,S3], S22)
        p("bytes_to_element", [S1,S2,S3], S23)
        p("arbitrary_element", [S1], S24)
        p("scalarmult(unknown-medium)", [S1,S2,S3,S22,S5medium,S6], S25)
        p("scalarmult(medium)", [S1,S2,S3,S23,S5medium,S6], S25)

if __name__ == "__main__":
    run()



import unittest
from binascii import hexlify
from pure25519.basic import Zero, ElementOfUnknownGroup
from pure25519.basic import xform_affine_to_extended, L, bytes_to_unknown_group_element

ORDERS = {1: "1", 2: "2", 4: "4", 8: "8",
          1*L: "1*L", 2*L: "2*L", 4*L: "4*L", 8*L: "8*L"}
def get_order(e):
    for o in sorted(ORDERS):
        if e.scalarmult(o) == Zero:
            return o

class Orders(unittest.TestCase):
    def assertElementsEqual(self, e1, e2, msg=None):
        self.assertEqual(hexlify(e1.to_bytes()), hexlify(e2.to_bytes()),
                         msg)

    def collect(self, e):
        values = set()
        for i in range(400):
            values.add(e.scalarmult(i).to_bytes())
        return values

    def test_orders(self):
        # all points should have an order that's listed in ORDERS. Test some
        # specific points. For low-order points, actually find the complete
        # subgroup and measure its size.

        p = Zero
        values = self.collect(p)
        self.assertEqual(len(values), 1)
        self.assertEqual(values, set([Zero.to_bytes()]))
        self.assertEqual(get_order(p), 1)

        # (0,-1) should be order 2
        p = ElementOfUnknownGroup(xform_affine_to_extended((0,-1)))
        values = self.collect(p)
        self.assertEqual(len(values), 2)
        self.assertEqual(values, set([Zero.to_bytes(), p.to_bytes()]))
        self.assertEqual(get_order(p), 2)

        # (..,26) is in the right group (order L)
        b = b"\x1a" + b"\x00"*31
        p = bytes_to_unknown_group_element(b)
        self.assertEqual(get_order(p), L)

        # (..,35) is maybe order 2*L
        b = b"\x23" + b"\x00"*31
        p = bytes_to_unknown_group_element(b)
        self.assertEqual(get_order(p), 2*L)

        # (..,48) is maybe order 4*L
        b = b"\x30" + b"\x00"*31
        p = bytes_to_unknown_group_element(b)
        self.assertEqual(get_order(p), 4*L)

        # (..,55) is maybe order 8*L
        b = b"\x37" + b"\x00"*31
        p = bytes_to_unknown_group_element(b)
        self.assertEqual(get_order(p), 8*L)



import timeit

def do(setup_statements, statement):
    # extracted from timeit.py
    t = timeit.Timer(stmt=statement,
                     setup="\n".join(setup_statements))
    # determine number so that 0.2 <= total time < 2.0
    for i in range(1, 10):
        number = 10**i
        x = t.timeit(number)
        if x >= 0.5:
            break
    return x / number

def abbrev(t):
    if t > 1.0:
        return "%.3fs" % t
    if t > 1e-3:
        return "%.2fms" % (t*1e3)
    return "%.2fus" % (t*1e6)

def p(name, setup_statements, statements):
    t = sorted([do(setup_statements, statements) for i in range(3)])
    print("%12s: %s (%s)" % (name,
                             abbrev(min(t)),
                             " ".join([abbrev(s) for s in t])))

def run():
    S1 = "import os; from pure25519 import spake2; pw=b'pw'; A=b'idA'; B=b'idB'"
    S2 = "sdata_U,X_s = spake2.start_U(pw, os.urandom, A, B)"
    S3 = "sdata_V,Y_s = spake2.start_V(pw, os.urandom, A, B)"
    S4 = "k = spake2.finish_U(sdata_U,Y_s)"

    print("speed_spake2")
    p("start", [S1], S2)
    p("finish", [S1, S2, S3], S4)

if __name__ == "__main__":
    run()



import os, unittest
from binascii import hexlify
from pure25519.basic import encodepoint
from pure25519.dh import dh_start, dh_finish

class DH(unittest.TestCase):
    def assertElementsEqual(self, e1, e2):
        self.assertEqual(hexlify(encodepoint(e1)), hexlify(encodepoint(e2)))
    def assertBytesEqual(self, e1, e2):
        self.assertEqual(hexlify(e1), hexlify(e2))

    def test_dh(self):
        for i in range(10):
            x,X_s = dh_start(os.urandom)
            y,Y_s = dh_start(os.urandom)
            z1 = dh_finish(x, Y_s)
            z2 = dh_finish(y, X_s)
            self.assertBytesEqual(z1, z2)



import timeit

def do(setup_statements, statement):
    # extracted from timeit.py
    t = timeit.Timer(stmt=statement,
                     setup="\n".join(setup_statements))
    # determine number so that 1.0 <= total time < 10.0
    for i in range(1, 10):
        number = 10**i
        x = t.timeit(number)
        if x >= 1.0:
            break
    return x / number

def abbrev(t):
    if t > 1.0:
        return "%.3fs" % t
    if t > 1e-3:
        return "%.2fms" % (t*1e3)
    return "%.2fus" % (t*1e6)

def p(name, setup_statements, statements):
    t = sorted([do(setup_statements, statements) for i in range(3)])
    print("%12s: %s (%s)" % (name,
                             abbrev(min(t)),
                             " ".join([abbrev(s) for s in t])))

def run():
    S1 = "from pure25519 import ed25519_oop; msg=b'hello world'"
    S2 = "sk,vk = ed25519_oop.create_keypair()"
    S3 = "sig = sk.sign(msg)"
    S4 = "vk.verify(sig, msg)"

    print("speed_ed25519")
    p("generate", [S1], S2)
    p("sign", [S1, S2], S3)
    p("verify", [S1, S2, S3], S4)

    S5 = "from pure25519 import eddsa"
    S6 = "h=eddsa.Hint(b'')"
    S7 = "eddsa.checkvalid(sig, msg, vk.vk_s)"

    p("Hint", [S5], S6)
    p("checkvalid", [S1,S2,S3,S5], S7)

if __name__ == "__main__":
    run()



from hashlib import sha256
from pure25519.basic import (arbitrary_element, bytes_to_element, Base,
                             random_scalar, password_to_scalar)

# a,b random. X=G*a+U*pw. Y=G*b+V*pw. Z1=(Y-V*pw)*a. Z2=(X-U*pw)*b

U = arbitrary_element(b"U")
V = arbitrary_element(b"V")

def _start(pw, entropy_f, blinding):
    a = random_scalar(entropy_f)
    pw_scalar = password_to_scalar(pw)
    X = Base.scalarmult(a).add(blinding.scalarmult(pw_scalar))
    X_s = X.to_bytes()
    return (a, pw_scalar), X_s

def _finish(start_data, Y_s, blinding):
    (a, pw_scalar) = start_data
    Y = bytes_to_element(Y_s) # rejects zero and non-group
    Z = Y.add(blinding.scalarmult(-pw_scalar)).scalarmult(a)
    return Z.to_bytes()


def start_U(pw, entropy_f, idA, idB, U=U, V=V):
    sdata, X_s = _start(pw, entropy_f, U)
    start_data = (sdata, pw, X_s, idA, idB, U, V)
    return start_data, X_s

def finish_U(start_data, Y_s):
    (sdata, pw, X_s, idA, idB, U, V) = start_data
    Z_s = _finish(sdata, Y_s, V)
    transcript = idA + idB + X_s + Y_s + Z_s + pw
    key = sha256(transcript).digest()
    return key


def start_V(pw, entropy_f, idA, idB, U=U, V=V):
    sdata, Y_s = _start(pw, entropy_f, V)
    start_data = (sdata, pw, Y_s, idA, idB, U, V)
    return start_data, Y_s

def finish_V(start_data, X_s):
    (sdata, pw, Y_s, idA, idB, U, V) = start_data
    Z_s = _finish(sdata, X_s, U)
    transcript = idA + idB + X_s + Y_s + Z_s + pw
    key = sha256(transcript).digest()
    return key




from __future__ import print_function
import sys
import unittest
from binascii import hexlify, unhexlify
import hashlib
from pure25519 import basic

if sys.version_info[0] == 2:
    def asbytes(b):
        """Convert array of integers to byte string"""
        return ''.join(chr(x) for x in b)
    def bit(h, i):
        """Return i'th bit of bytestring h"""
        return (ord(h[i//8]) >> (i%8)) & 1
else:
    asbytes = bytes
    def bit(h, i):
        return (h[i//8] >> (i%8)) & 1

b = 256 # =32*8
def H(m):
    return hashlib.sha512(m).digest()

class Compare(unittest.TestCase):
    # I want to replace some very manual (and slow) loops with builtins.
    # These tests are to ensure that the replacements work the same way

    # Hint turns bytes into a 512-bit integer, little-endian (the first byte
    # of H(m) is used for the low-order 8 bits of the integer, with the LSB
    # of H(m)[0] used as the LSB of the integer).
    def orig_Hint(self, m):
        h = H(m)
        return sum(2**i * bit(h,i) for i in range(2*b))
    def new_Hint(self, m):
        h = H(m)
        return int(hexlify(h[::-1]), 16)
    def test_Hint(self):
        for i in range(200):
            m = str(i).encode("ascii")
            self.assertEqual(self.orig_Hint(m), self.new_Hint(m))

    def orig_decodepoint_sum(self, s):
        # the original. 802us.
        y = sum(2**i * bit(s,i) for i in range(0,b-1))
        #print("%064x" % y)
        return y

    def new_decodepoint_sum_1(self, s):
        # the range(0,b-1) means we don't use the MSB of the last byte, so
        # the high-order bit of the int will be zero. In this form, we clamp
        # the int. 337us
        unclamped = int(hexlify(s[:32][::-1]), 16)
        clamp = (1 << 255) - 1
        clamped = unclamped & clamp
        return clamped

    def new_decodepoint_sum_2(self, s):
        # In this form, we remap the last byte before conversion. 340us
        s = s[:32]
        clamped_byte = unhexlify("%02x" % (int(hexlify(s[-2:]), 16) & 0x7f))
        clamped_s = s[:-1] + clamped_byte
        clamped = int(hexlify(clamped_s[::-1]), 16)
        #print("%064x" % clamped)
        return clamped

    def test_decodepoint_sum(self):
        for i in range(200):
            #print()
            s = H(str(i).encode("ascii"))[:32]
            self.assertEqual(self.orig_decodepoint_sum(s),
                             self.new_decodepoint_sum_1(s))
            self.assertEqual(self.new_decodepoint_sum_1(s),
                             self.new_decodepoint_sum_2(s))

    def orig_decodepoint_2(self, s):
        # already sped up once
        unclamped = int(hexlify(s[:32][::-1]), 16)
        clamp = (1 << 255) - 1
        y = unclamped & clamp
        x = basic.xrecover(y)
        if x & 1 != bit(s,b-1): x = basic.Q-x
        P = [x,y]
        if not basic.isoncurve(P):
            raise Exception("decoding point that is not on curve")
        return P

    def new_decodepoint_2(self, s):
        # already sped up once
        unclamped = int(hexlify(s[:32][::-1]), 16)
        clamp = (1 << 255) - 1
        y = unclamped & clamp
        x = basic.xrecover(y)
        if bool(x & 1) != bool(unclamped & (1<<255)): x = basic.Q-x
        P = [x,y]
        if not basic.isoncurve(P):
            raise Exception("decoding point that is not on curve")
        return P

    def test_decodepoint_2(self):
        for i in range(200):
            P_s = basic.Base.scalarmult(i).to_bytes()
            self.assertEqual(self.orig_decodepoint_2(P_s),
                             self.new_decodepoint_2(P_s))

    def orig_decodeint(self, s):
        return sum(2**i * bit(s,i) for i in range(0,b))
    def new_decodeint(self, s):
        return int(hexlify(s[:32][::-1]), 16)

    def test_decodeint(self):
        for i in range(200):
            s = H(str(i).encode("ascii"))[:32]
            self.assertEqual(self.orig_decodeint(s),
                             self.new_decodeint(s))

    def orig_signature_sum(self, sk):
        h = H(sk)
        a = 2**(b-2) + sum(2**i * bit(h,i) for i in range(3,b-2))
        #print("%064x" % a)
        return a

    def new_signature_sum(self, sk):
        h = H(sk)
        unclamped = int(hexlify(h[:32][::-1]), 16)
        # top byte is      01xxxxxx
        # middle bytes are xxxxxxxx
        # bottom byte is   xxxxx000
        top_reset_clamp = (0x40 << (31*8)) - 1
        bottom_reset_clamp = ((1 << (32*8)) - 1) ^ 0x7
        set_clamp = 0x40 << (31*8)
        clamped = (unclamped & top_reset_clamp & bottom_reset_clamp) | set_clamp
        #print("%064x" % unclamped)
        #print("%064x" % set_clamp)
        #print("%064x" % top_reset_clamp)
        #print("%064x" % bottom_reset_clamp)
        #print("%064x" % clamped)
        return clamped

    def test_signature_sum(self):
        for i in range(200):
            #print()
            sk = H(str(i).encode("ascii"))[:32]
            self.assertEqual(self.orig_signature_sum(sk),
                             self.new_signature_sum(sk))

    def orig_encodeint(self, y):
        bits = [(y >> i) & 1 for i in range(b)]
        e = [(sum([bits[i * 8 + j] << j for j in range(8)]))
                                        for i in range(b//8)]
        return asbytes(e)

    def new_encodeint(self, y):
        assert 0 <= y < 2**256
        return unhexlify("%064x" % y)[::-1]

    def test_encodeint(self):
        for i in range(200):
            s = H(str(i).encode("ascii"))[:32]
            sint = self.orig_decodeint(s)
            self.assertEqual(self.orig_encodeint(sint),
                             self.new_encodeint(sint))

    def orig_encodepoint(self, P):
        x = P[0]
        y = P[1]
        bits = [(y >> i) & 1 for i in range(b - 1)] + [x & 1]
        e = [(sum([bits[i * 8 + j] << j for j in range(8)]))
                                        for i in range(b//8)]
        return asbytes(e)

    def new_encodepoint(self, P):
        x = P[0]
        y = P[1]
        # MSB of output equals x.b0 (=x&1)
        # rest of output is little-endian y
        assert 0 <= y < (1<<255) # always < 0x7fff..ff
        if x & 1:
            y += 1<<255
        return unhexlify("%064x" % y)[::-1]

    def test_encodepoint(self):
        for i in range(200):
            P = basic.xform_extended_to_affine(basic.Base.scalarmult(i).XYTZ)
            self.assertEqual(self.orig_encodepoint(P),
                             self.new_encodepoint(P))



from __future__ import print_function
import sys
import unittest
import time
from binascii import hexlify, unhexlify
from pure25519 import ed25519_oop as ed25519
from pure25519 import _ed25519 as raw

if sys.version_info[0] == 3:
    def int2byte(i):
        return bytes((i,))
else:
    int2byte = chr

def flip_bit(s, bit=0, in_byte=-1):
    as_bytes = [ord(b) if isinstance(b, str) else b for b in s]
    as_bytes[in_byte] = as_bytes[in_byte] ^ (0x01<<bit)
    return  b"".join([int2byte(b) for b in as_bytes])

# the pure-python demonstration code (on my 2010 MacBookPro) takes 5s to
# generate a public key, 9s to sign, 14s to verify

# the SUPERCOP-ref version we use takes 2ms for keygen, 2ms to sign, and 7ms
# to verify

class Basic(unittest.TestCase):
    timer = None
    def log(self, msg):
        return
        now = time.time()
        if self.timer is None:
            self.timer = now
        else:
            elapsed = now - self.timer
            self.timer = now
            print(" (%f elapsed)" % elapsed)
        print(msg)

    def test_constants(self):
        # the secret key we get from raw.keypair() are 64 bytes long, and
        # are mostly the output of a sha512 call. The first 32 bytes are the
        # private exponent (random, with a few bits stomped).
        self.assertEqual(raw.SECRETKEYBYTES, 64)
        # the public key is the encoded public point
        self.assertEqual(raw.PUBLICKEYBYTES, 32)
        self.assertEqual(raw.SIGNATUREKEYBYTES, 64)

    def test_raw(self):
        sk_s = b"\x00" * 32 # usually urandom(32)
        vk_s, skvk_s = raw.publickey(sk_s)
        self.assertEqual(len(vk_s), 32)
        exp_vks = unhexlify("3b6a27bcceb6a42d62a3a8d02a6f0d73"
                            "653215771de243a63ac048a18b59da29")
        self.assertEqual(vk_s, exp_vks)
        self.assertEqual(skvk_s[:32], sk_s)
        self.assertEqual(skvk_s[32:], vk_s)
        msg = b"hello world"
        msg_and_sig = raw.sign(msg, skvk_s)
        sig = msg_and_sig[:-len(msg)]
        self.assertEqual(len(sig), 64)
        exp_sig = unhexlify("b0b47780f096ae60bfff8d8e7b19c36b"
                            "321ae6e69cca972f2ff987ef30f20d29"
                            "774b53bae404485c4391ddf1b3f37aaa"
                            "8a9747f984eb0884e8aa533386e73305")
        self.assertEqual(sig, exp_sig)
        ret = raw.open(sig+msg, vk_s) # don't raise exception
        self.assertEqual(ret, msg)
        self.assertRaises(raw.BadSignatureError,
                          raw.open, sig+msg+b".. NOT!", vk_s)
        self.assertRaises(raw.BadSignatureError,
                          raw.open, sig+flip_bit(msg), vk_s)
        self.assertRaises(raw.BadSignatureError,
                          raw.open, sig+msg, flip_bit(vk_s))
        self.assertRaises(raw.BadSignatureError,
                          raw.open, sig+msg, flip_bit(vk_s, in_byte=2))
        self.assertRaises(raw.BadSignatureError,
                          raw.open, flip_bit(sig)+msg, vk_s)
        self.assertRaises(raw.BadSignatureError,
                          raw.open, flip_bit(sig, in_byte=33)+msg, vk_s)

    def test_keypair(self):
        sk, vk = ed25519.create_keypair()
        self.assertTrue(isinstance(sk, ed25519.SigningKey), sk)
        self.assertTrue(isinstance(vk, ed25519.VerifyingKey), vk)
        sk2, vk2 = ed25519.create_keypair()
        self.assertNotEqual(hexlify(sk.to_bytes()), hexlify(sk2.to_bytes()))

        # you can control the entropy source
        def not_so_random(length):
            return b"4"*length
        sk1, vk1 = ed25519.create_keypair(entropy=not_so_random)
        self.assertEqual(sk1.to_ascii(encoding="base64"),
                         b"NDQ0NDQ0NDQ0NDQ0NDQ0NDQ0NDQ0NDQ0NDQ0NDQ0NDQ")
        self.assertEqual(vk1.to_ascii(encoding="base64"),
                         b"6yzxO/euOl9hQWih+wknLTl3HsS4UjcngV5GbK+O4WM")
        sk2, vk2 = ed25519.create_keypair(entropy=not_so_random)
        self.assertEqual(sk1.to_ascii(encoding="base64"),
                         sk2.to_ascii(encoding="base64"))
        self.assertEqual(vk1.to_ascii(encoding="base64"),
                         vk2.to_ascii(encoding="base64"))


    def test_publickey(self):
        seed = unhexlify("4ba96b0b5303328c7405220598a587c4"
                         "acb06ed9a9601d149f85400195f1ec3d")
        sk = ed25519.SigningKey(seed)
        self.assertEqual(hexlify(sk.to_bytes()),
                         (b"4ba96b0b5303328c7405220598a587c4"
                          b"acb06ed9a9601d149f85400195f1ec3d"
                          b"a66d161e090652b054740748f059f92a"
                          b"5b731f1c27b05571f6d942e4f8b7b264"))
        self.assertEqual(hexlify(sk.to_seed()),
                         (b"4ba96b0b5303328c7405220598a587c4"
                          b"acb06ed9a9601d149f85400195f1ec3d"))
        self.assertRaises(ValueError,
                          ed25519.SigningKey, b"wrong length")
        sk2 = ed25519.SigningKey(seed)
        self.assertEqual(sk, sk2)

    def test_OOP(self):
        sk_s = unhexlify("4ba96b0b5303328c7405220598a587c4"
                         "acb06ed9a9601d149f85400195f1ec3d"
                         "a66d161e090652b054740748f059f92a"
                         "5b731f1c27b05571f6d942e4f8b7b264")
        sk = ed25519.SigningKey(sk_s)
        self.assertEqual(len(sk.to_bytes()), 64)
        self.assertEqual(sk.to_bytes(), sk_s)

        sk2_seed = unhexlify("4ba96b0b5303328c7405220598a587c4"
                             "acb06ed9a9601d149f85400195f1ec3d")
        sk2 = ed25519.SigningKey(sk2_seed)
        self.assertEqual(sk2.to_bytes(), sk.to_bytes())

        vk = sk.get_verifying_key()
        self.assertEqual(len(vk.to_bytes()), 32)
        exp_vks = unhexlify("a66d161e090652b054740748f059f92a"
                            "5b731f1c27b05571f6d942e4f8b7b264")
        self.assertEqual(vk.to_bytes(), exp_vks)
        self.assertEqual(ed25519.VerifyingKey(vk.to_bytes()), vk)
        msg = b"hello world"
        sig = sk.sign(msg)
        self.assertEqual(len(sig), 64)
        exp_sig = unhexlify("6eaffe94f2972b35158b6aaa9b69c1da"
                            "97f0896aca29c41b1dd7b32e6c9e2ff6"
                            "76fc8d8b034709cdcc37d8aeb86bebfb"
                            "173ace3c319e211ea1d7e8d8884c1808")
        self.assertEqual(sig, exp_sig)
        self.assertEqual(vk.verify(sig, msg), None) # also, don't throw
        self.assertRaises(ed25519.BadSignatureError,
                          vk.verify, sig, msg+b".. NOT!")

    def test_object_identity(self):
        sk1_s = unhexlify("ef32972ae3f1252a5aa1395347ea008c"
                          "bd2fed0773a4ea45e2d2d06c8cf8fbd4"
                          "c024601a9c5b854fb100ff3116cf4f22"
                          "a311565f027391cb49d3bbe11c44399d")
        sk2_s = unhexlify("3d550c158900b4c2922b6656d2f80572"
                          "89de4ee65043745179685ae7d29b944d"
                          "672b8a2cb23f9e75e1d46ce249cd9c04"
                          "68f816f1c734a102822b60e18b41eacd")
        sk1a = ed25519.SigningKey(sk1_s)
        sk1b = ed25519.SigningKey(sk1_s)
        vk1a = sk1a.get_verifying_key()
        vk1b = sk1b.get_verifying_key()
        sk2 = ed25519.SigningKey(sk2_s)
        vk2 = sk2.get_verifying_key()
        self.assertEqual(sk1a, sk1b)
        self.assertNotEqual(sk1a, sk2)
        self.assertEqual(vk1a, vk1b)
        self.assertNotEqual(vk1a, vk2)

        self.assertNotEqual(sk2, b"not a SigningKey")
        self.assertNotEqual(vk2, b"not a VerifyingKey")

    def test_prefix(self):
        sk1,vk1 = ed25519.create_keypair()
        PREFIX = b"private0-"
        p = sk1.to_bytes(PREFIX)
        # that gives us a binary string with a prefix
        self.assertTrue(p[:len(PREFIX)] == PREFIX, repr(p))
        sk2 = ed25519.SigningKey(p, prefix=PREFIX)
        self.assertEqual(sk1, sk2)
        self.assertEqual(repr(sk1.to_bytes()), repr(sk2.to_bytes()))
        self.assertRaises(ed25519.BadPrefixError,
                          ed25519.SigningKey, p, prefix=b"WRONG-")
        # SigningKey.to_seed() can do a prefix too
        p = sk1.to_seed(PREFIX)
        self.assertTrue(p[:len(PREFIX)] == PREFIX, repr(p))
        sk3 = ed25519.SigningKey(p, prefix=PREFIX)
        self.assertEqual(sk1, sk3)
        self.assertEqual(repr(sk1.to_bytes()), repr(sk3.to_bytes()))
        self.assertRaises(ed25519.BadPrefixError,
                          ed25519.SigningKey, p, prefix=b"WRONG-")

        # verifying keys can do this too
        PREFIX = b"public0-"
        p = vk1.to_bytes(PREFIX)
        self.assertTrue(p.startswith(PREFIX), repr(p))
        vk2 = ed25519.VerifyingKey(p, prefix=PREFIX)
        self.assertEqual(vk1, vk2)
        self.assertEqual(repr(vk1.to_bytes()), repr(vk2.to_bytes()))
        self.assertRaises(ed25519.BadPrefixError,
                          ed25519.VerifyingKey, p, prefix=b"WRONG-")

        # and signatures
        PREFIX = b"sig0-"
        p = sk1.sign(b"msg", PREFIX)
        self.assertTrue(p.startswith(PREFIX), repr(p))
        vk1.verify(p, b"msg", PREFIX)
        self.assertRaises(ed25519.BadPrefixError,
                          vk1.verify, p, b"msg", prefix=b"WRONG-")

    def test_ascii(self):
        b2a = ed25519.to_ascii
        a2b = ed25519.from_ascii
        for prefix in ("", "prefix-"):
            for length in range(0, 100):
                b1 = b"a"*length
                for base in ("base64", "base32", "base16", "hex"):
                    a = b2a(b1, prefix, base)
                    b2 = a2b(a, prefix, base)
                    self.assertEqual(b1, b2)

    def test_encoding(self):
        sk_s = b"\x88" * 32 # usually urandom(32)
        sk1 = ed25519.SigningKey(sk_s)
        vk1 = sk1.get_verifying_key()

        def check1(encoding, expected):
            PREFIX = "private0-"
            p = sk1.to_ascii(PREFIX, encoding)
            self.assertEqual(p, expected)
            sk2 = ed25519.SigningKey(p, prefix=PREFIX, encoding=encoding)
            self.assertEqual(repr(sk1.to_bytes()), repr(sk2.to_bytes()))
            self.assertEqual(sk1, sk2)
        check1("base64", b"private0-iIiIiIiIiIiIiIiIiIiIiIiIiIiIiIiIiIiIiIiIiIg")
        check1("base32", b"private0-rceirceirceirceirceirceirceirceirceirceirceirceircea")
        check1("hex", b"private0-8888888888888888888888888888888888888888888888888888888888888888")

        def check2(encoding, expected):
            PREFIX="public0-"
            p = vk1.to_ascii(PREFIX, encoding)
            self.assertEqual(p, expected)
            vk2 = ed25519.VerifyingKey(p, prefix=PREFIX, encoding=encoding)
            self.assertEqual(repr(vk1.to_bytes()), repr(vk2.to_bytes()))
            self.assertEqual(vk1, vk2)
        check2("base64", b"public0-skkdlQKuKGMKK6yy4MdFEP/N0yjDNP8+E5PnWy0x59w")
        check2("base32", b"public0-wjer3ficvyuggcrlvszobr2fcd743uziym2p6pqtsptvwljr47oa")
        check2("hex", b"public0-b2491d9502ae28630a2bacb2e0c74510ffcdd328c334ff3e1393e75b2d31e7dc")

        def check3(encoding, expected):
            msg = b"msg"
            PREFIX="sig0-"
            sig = sk1.sign(msg, PREFIX, encoding)
            self.assertEqual(sig, expected)
            vk1.verify(sig, msg, PREFIX, encoding)
        check3("base64", b"sig0-MNfdUir6tMlaYQ+/p8KANJ5d+bk8g2al76v5MeJCo6RiywxURda3sU580CyiW2FBG/Q7kDRswgYqxbkQw3o5CQ")
        check3("base32", b"sig0-gdl52urk7k2mswtbb672pquagspf36nzhsbwnjppvp4tdyscuosgfsymkrc5nn5rjz6nalfclnqucg7uhoidi3gcayvmloiqyn5dsci")
        check3("hex", b"sig0-30d7dd522afab4c95a610fbfa7c280349e5df9b93c8366a5efabf931e242a3a462cb0c5445d6b7b14e7cd02ca25b61411bf43b90346cc2062ac5b910c37a3909")


if __name__ == '__main__':
    unittest.main()



from . import eddsa

# adapt pure25519/ed25519.py to behave like (C/glue) ed25519/_ed25519.py, so
# ed25519_oop.py doesn't have to change

# ed25519 secret/private/signing keys can be built from a 32-byte random seed
# (clamped and treated as a scalar). The public/verifying key is a 32-byte
# encoded group element. Signing requires both, so a common space/time
# tradeoff is to glue the two together and call the 64-byte combination the
# "secret key". Signatures are 64 bytes (one encoded group element and one
# encoded scalar). The NaCl code prefers to glue the signature to the
# message, rather than pass around detacted signatures.
#
# for clarity, we use the following notation:
#  seed: 32-byte secret random seed (unclamped)
#  sk: = seed
#  vk: 32-byte verifying key (encoded group element)
#  seed+vk: 64-byte glue thing (sometimes stored as secret key)
#  sig: 64-byte detached signature (R+S)
#  sig+msg: signature concatenated to message

# that glue provides:
#  SECRETKEYBYTES=64, PUBLICKEYBYTES=32, SIGNATUREBYTES=64
#  (vk,seed+vk)=publickey(seed)
#  sig+msg = sign(msg, seed+vk)
#  msg = open(sig+msg, vk) # or raise BadSignatureError

# pure25519/ed25519.py provides:
#  vk = publickey(sk)
#  sig = signature(msg, sk or sk+vk, vk)
#  bool = checkvalid(sig, msg, vk)

class BadSignatureError(Exception):
    pass

SECRETKEYBYTES = 64
PUBLICKEYBYTES = 32
SIGNATUREKEYBYTES = 64

def publickey(seed32):
    assert len(seed32) == 32
    vk32 = eddsa.publickey(seed32)
    return vk32, seed32+vk32

def sign(msg, skvk):
    assert len(skvk) == 64
    sk = skvk[:32]
    vk = skvk[32:]
    sig = eddsa.signature(msg, sk, vk)
    return sig+msg

def open(sigmsg, vk):
    assert len(vk) == 32
    sig = sigmsg[:64]
    msg = sigmsg[64:]
    try:
        valid = eddsa.checkvalid(sig, msg, vk)
    except ValueError as e:
        raise BadSignatureError(e)
    except Exception as e:
        if str(e) == "decoding point that is not on curve":
            raise BadSignatureError(e)
        raise
    if not valid:
        raise BadSignatureError()
    return msg



from pure25519.basic import random_scalar, Base, bytes_to_element
from hashlib import sha256

# In practice, you should use the Curve25519 function, which is better in
# every way. But this is an example of what Diffie-Hellman looks like.

def dh_start(entropy_f):
    x = random_scalar(entropy_f)
    X = Base.scalarmult(x)
    return x,X.to_bytes()

def dh_finish(x, Y_s):
    Y = bytes_to_element(Y_s)
    XY = Y.scalarmult(x)
    return sha256(XY.to_bytes()).digest()



import unittest
import random
from binascii import hexlify
from pure25519.basic import (Q, L, B, ElementOfUnknownGroup,
                             arbitrary_element, password_to_scalar,
                             bytes_to_element, bytes_to_unknown_group_element,
                             _add_elements_nonunfied, add_elements, encodepoint,
                             xform_extended_to_affine, xform_affine_to_extended)
from pure25519.basic import Base, Element, Zero
from pure25519.slow_basic import (slow_add_affine, scalarmult_affine,
                                  scalarmult_affine_to_extended)

class Basic(unittest.TestCase):
    def assertElementsEqual(self, e1, e2, msg=None):
        self.assertEqual(hexlify(e1.to_bytes()), hexlify(e2.to_bytes()),
                         msg)
    def assertBytesEqual(self, e1, e2, msg=None):
        self.assertEqual(hexlify(e1), hexlify(e2), msg)

    def test_arbitrary_element(self):
        for i in range(20):
            seed = str(i).encode("ascii")
            e = arbitrary_element(seed)
            e2 = arbitrary_element(seed)
            self.assertElementsEqual(e, e2)

    def test_password_to_scalar(self):
        for i in range(20):
            seed = str(i).encode("ascii")
            s = password_to_scalar(seed)
            s2 = password_to_scalar(seed)
            self.assertEqual(s, s2)

    def test_scalarmult(self):
        Bsm = Base.scalarmult
        e0 = Bsm(0)
        e1 = Bsm(1)
        e2 = Bsm(2)
        e5 = Bsm(5)
        e10 = Bsm(10)
        e15 = Bsm(15)
        em5 = Bsm(-5)
        self.assertElementsEqual(e0.add(e0), e0)
        self.assertElementsEqual(e1, Base)
        self.assertElementsEqual(e0.add(e1), e1)
        self.assertElementsEqual(Bsm(-5), Bsm(-5 % L))
        # there was a bug, due to the add inside scalarmult being
        # non-unified, which caused e0.scalarmult(N) to not equal e0
        self.assertElementsEqual(e0.scalarmult(5), e0)
        self.assertElementsEqual(e2.scalarmult(0), e0)
        self.assertElementsEqual(e1.add(e1), e2)
        self.assertElementsEqual(e5.add(e5), e10)
        self.assertElementsEqual(e5.add(e10), e15)
        self.assertElementsEqual(e2.scalarmult(5), e10)
        self.assertElementsEqual(e15.add(em5), e10)
        self.assertElementsEqual(e5.add(em5), e0)

        sm2 = scalarmult_affine
        e = {}
        for i in range(-50, 100):
            e[i] = Bsm(i)
            self.assertElementsEqual(Element(xform_affine_to_extended(sm2(B, i))), e[i])
            self.assertElementsEqual(Element(scalarmult_affine_to_extended(B, i)),
                                     e[i])

        for i in range(20,30):
            for j in range(-10,10):
                x1 = e[i].add(e[j])
                x2 = Bsm(i+j)
                self.assertElementsEqual(x1, x2, (x1,x2,i,j))
                x3 = Element(xform_affine_to_extended(sm2(B, i+j)))
                self.assertElementsEqual(x1, x3, (x1,x3,i,j))

    def test_orders(self):
        # the point (0,1) is the identity, and has order 1
        p0 = xform_affine_to_extended((0,1))
        # p0+p0=p0
        # p0+anything=anything

        # The point (0,-1) has order 2
        p2 = xform_affine_to_extended((0,-1))
        p3 = add_elements(p2, p2) # p3=p2+p2=p0
        p4 = add_elements(p3, p2) # p4=p3+p2=p2
        p5 = add_elements(p4, p2) # p5=p4+p2=p0
        self.assertBytesEqual(encodepoint(xform_extended_to_affine(p3)),
                              encodepoint(xform_extended_to_affine(p0)))
        self.assertBytesEqual(encodepoint(xform_extended_to_affine(p4)),
                              encodepoint(xform_extended_to_affine(p2)))
        self.assertBytesEqual(encodepoint(xform_extended_to_affine(p5)),
                              encodepoint(xform_extended_to_affine(p0)))

        # now same thing, but with Element
        p0 = ElementOfUnknownGroup(xform_affine_to_extended((0,1)))
        self.assertElementsEqual(p0, Zero)
        p2 = ElementOfUnknownGroup(xform_affine_to_extended((0,-1)))
        p3 = p2.add(p2)
        p4 = p3.add(p2)
        p5 = p4.add(p2)
        self.assertElementsEqual(p3, p0)
        self.assertElementsEqual(p4, p2)
        self.assertElementsEqual(p5, p0)
        self.assertFalse(isinstance(p3, Element))
        self.assertFalse(isinstance(p4, Element))
        self.assertFalse(isinstance(p5, Element))

        # and again, with .scalarmult instead of .add
        p3 = p2.scalarmult(2) # p3=2*p2=p0
        p4 = p2.scalarmult(3) # p4=3*p2=p2
        p5 = p2.scalarmult(4) # p5=4*p2=p0
        self.assertElementsEqual(p3, p0) # TODO: failing
        self.assertElementsEqual(p4, p2)
        self.assertElementsEqual(p5, p0)
        self.assertFalse(isinstance(p3, Element))
        self.assertFalse(isinstance(p4, Element))
        self.assertFalse(isinstance(p5, Element))

        # I don't know if there are points of order 4.

    def test_add(self):
        e = []
        for i in range(100):
            seed = str(i).encode("ascii")
            s = password_to_scalar(seed)
            e.append(Base.scalarmult(s))
        for i in range(100):
            x = random.choice(e)
            y = random.choice(e)

            sum1 = element_from_affine(
                slow_add_affine(xform_extended_to_affine(x.XYTZ),
                                xform_extended_to_affine(y.XYTZ)))
            sum2 = x.add(y)
            self.assertElementsEqual(sum1, sum2)
            if x != y:
                sum3 = Element(_add_elements_nonunfied(x.XYTZ, y.XYTZ))
                self.assertElementsEqual(sum1, sum3)

    def test_bytes_to_element(self):
        b = encodepoint((0,1)) # order 1, aka Zero
        self.assertRaises(ValueError, bytes_to_element, b)
        p = bytes_to_unknown_group_element(b)
        self.assertFalse(isinstance(p, Element))
        self.assertIs(p, Zero)

        b = encodepoint((0,-1%Q)) # order 2
        self.assertRaises(ValueError, bytes_to_element, b)
        p = bytes_to_unknown_group_element(b)
        self.assertFalse(isinstance(p, Element))

        # (..,26) is in the right group
        b = b"\x1a" + b"\x00"*31
        p = bytes_to_element(b)
        self.assertTrue(isinstance(p, Element))


def element_from_affine(P):
    return Element(xform_affine_to_extended(P))



#!/usr/bin/env python

from __future__ import print_function
import sys, unittest
from distutils.core import setup, Command

class Test(Command):
    description = "run tests"
    user_options = []
    def initialize_options(self):
        pass
    def finalize_options(self):
        pass
    def run(self):
        test = unittest.defaultTestLoader.discover("pure25519")
        runner = unittest.TextTestRunner(verbosity=2)
        result = runner.run(test)
        sys.exit(not result.wasSuccessful())

class KnownAnswerTest(Test):
    description = "run known-answer-tests"
    def run(self):
        test = unittest.defaultTestLoader.loadTestsFromName("pure25519.do_ed25519_kat")
        runner = unittest.TextTestRunner(verbosity=2)
        result = runner.run(test)
        sys.exit(not result.wasSuccessful())

class Speed(Test):
    description = "run benchmark suite"
    def run(self):
        from pure25519 import speed_basic, speed_ed25519, speed_dh, speed_spake2
        speed_basic.run()
        speed_ed25519.run()
        speed_dh.run()
        speed_spake2.run()

setup(name="pure25519",
      version="0.0.1", # not for publication
      description="pure-python curve25519/ed25519 routines",
      author="Brian Warner",
      author_email="warner-python-pure25519@lothar.com",
      license="MIT",
      url="https://github.com/warner/python-pure25519",
      packages=["pure25519"],
      package_dir={"pure25519": "pure25519"},
      cmdclass={"test": Test, "speed": Speed, "test_kat": KnownAnswerTest},
      )




from pure25519.basic import (inv, d, Q, L,
                             xform_extended_to_affine,
                             scalarmult_element,
                             xform_affine_to_extended,
                             double_element, _add_elements_nonunfied)

# Affine Coordinates: only here to compare against faster versions

def slow_add_affine(A,B): # affine->affine
    # Complete: works even when A==B. Slow: 50x slower than extended
    x1 = A[0]
    y1 = A[1]
    x2 = B[0]
    y2 = B[1]
    x3 = (x1*y2+x2*y1) * inv(1+d*x1*x2*y1*y2)
    y3 = (y1*y2+x1*x2) * inv(1-d*x1*x2*y1*y2)
    return (x3 % Q,y3 % Q)

def slow_scalarmult_affine(A,e): # affine->affine
    e = e % L
    if e == 0: return [0,1]
    B = slow_scalarmult_affine(A,e//2)
    B = slow_add_affine(B,B)
    if e & 1: B = slow_add_affine(B,A)
    return B

# other functions that are only here for speed comparisons

def scalarmult_affine(pt, e): # affine->affine
    e = e % L
    return xform_extended_to_affine(
            scalarmult_element(
             xform_affine_to_extended(pt),
            e))

def scalarmult_affine_to_extended(pt, n): # affine->extended
    assert len(pt) == 2 # affine
    n = n % L
    if n==0: return xform_affine_to_extended((0,1))
    xpt = xform_affine_to_extended(pt) # so Z=1
    return _scalarmult_affine_to_extended_inner(xpt, n)

def _scalarmult_affine_to_extended_inner(xpt, n):
    if n==0: return xform_affine_to_extended((0,1))
    _ = double_element(_scalarmult_affine_to_extended_inner(xpt, n>>1))
    return _add_elements_nonunfied(_, xpt) if n&1 else _
